@@ -1,11 +1,11 @@
 """ metdata constants """
 
 
-DWD_FOLDER_MAIN = './dwd_data'
+DWD_FOLDER_MAIN = "./dwd_data"
 DWD_FOLDER_STATION_DATA = "station_data"
 DWD_FILE_STATION_DATA = "dwd_station_data"
-ARCHIVE_FORMAT = '.zip'
-DATA_FORMAT = '.csv'
+ARCHIVE_FORMAT = ".zip"
+DATA_FORMAT = ".csv"
 H5_FORMAT = ".h5"
 DATE_FORMAT = "%Y%m%d"
 STATION_ID_REGEX = r"(?<!\d)\d{5}(?!\d)"
